/* igmp_list.c - */

/* Copyright (c) 2013, TeamF1, Inc. */ 
 
/* Maintains a list of all downstream clients and mulicast groups to which they
 * are listening.
 *
 * This list can be used to determine, when a leave message comes, whether
 * there are anymore downstream clients listening on that group. Not very
 * reliable, as not always does a client send a leave message when it is leaving
 * the group.
 */


#include "defs.h"

struct igmp_group_node {
    uint32_t group;
    uint32_t listener_ip;

    struct igmp_group_node *next;
};

static struct igmp_group_node igmp_group_head;

void init_igmp_list (void)
    {
    memset (&igmp_group_head, 0, sizeof(igmp_group_head));
    
    igmp_group_head.group = 0;
    igmp_group_head.listener_ip = 0;
    igmp_group_head.next = NULL;

    return;
    }

int add_group_to_list (uint32_t group, uint32_t ip)
    {
    struct igmp_group_node *pGroup = &igmp_group_head;
    struct igmp_group_node *pGroupPrev = NULL;

    while (pGroup != NULL) {
        if (pGroup->group == group &&
            pGroup->listener_ip == ip)
            break;

        pGroupPrev = pGroup;
        pGroup = pGroup->next;
    }

    if (pGroup != NULL) {
        log(LOG_INFO, "[%s] Group-Ip %u-%u already present. \n",__func__,group,ip);
        return 0;
    }
        
    pGroup = (struct igmp_group_node *)malloc (sizeof(struct igmp_group_node));
    if (pGroup == NULL)
        {
        /* fatal. cannot allocate memory */
        log(LOG_WARNING, "[%s] Failed to allocate memory. \n",__func__);
        return 1;
        }
       
    memset (pGroup, 0, sizeof(*pGroup));
    pGroup->group = group;
    pGroup->listener_ip = ip;
    pGroup->next = NULL;
    pGroupPrev->next = pGroup;

    return 0;
    }

int del_group_from_list (uint32_t group, uint32_t ip)
    {
    struct igmp_group_node *pGroup = &igmp_group_head;
    struct igmp_group_node *pGroupPrev = NULL;

    while (pGroup != NULL) {
        if (pGroup->group == group &&
            pGroup->listener_ip == ip)
            break;
        
        pGroupPrev = pGroup;
        pGroup = pGroup->next;
    }

    if (pGroup == NULL)
        {
        /* error error . group not present. */
        log(LOG_WARNING, "[%s] Trying to delete a group-ip %u-%u not present "
            " in the list \n",__func__,group,ip);
        return 1;
        }

    pGroupPrev->next = pGroup->next;

    free (pGroup);

    return 0;
    }

int check_group_in_list (uint32_t group)
    {
    struct igmp_group_node *pGroup = &igmp_group_head;

    while (pGroup != NULL) {
        if (pGroup->group == group)
            break;

        pGroup = pGroup->next;
    }

    if (pGroup == NULL) {
        log(LOG_INFO, "[%s] Group %u not present. \n",__func__,group);
        return 1;
    }

    return 0;
    }

void flush_group_in_list (uint32_t group)
    {
    struct igmp_group_node *pGroup = &igmp_group_head;
    struct igmp_group_node *pGroupPrev = NULL;

    while (pGroup != NULL) {
        if (pGroup->group == group) {
            pGroupPrev->next = pGroup->next;
            free (pGroup);
            pGroup = pGroupPrev->next;
            continue;
        }

        pGroupPrev = pGroup;
        pGroup = pGroup->next;
    }

    log(LOG_INFO, "[%s] Group-IP list flushed of group %u. \n",__func__,group);
    return;
    }
