/* upnpConfig.c - upnp config utility */

/* Copyright (c) 2007, TeamF1, Inc. */

/*
modification history
--------------------
01a,31Jan08,sau  written.
*/

/*
DESCRIPTION

This file creates the configuration file and calls binary for upnp. 
*/

/* includes */
 
#include <adaptos.h>
#include <umi.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdint.h>
#include <sys/un.h>
#include <errno.h>
#include <unistd.h>

/* defines */

#define UPNP_FILE_NAME "/tmp/upnp-socket"
#define ROUTE_ADD      "route add -net 239.0.0.0 netmask 255.0.0.0 "
#define ROUTE_DEL      "route del -net 239.0.0.0 netmask 255.0.0.0 "
#define KILL_UPNP      "/usr/bin/killall upnpd"

/* typedefs */


/* globals */


/* locals */

ADP_MOD_ID(UMI_COMP_UPNP);

/* imports */

/* declarations */

typedef struct apeEntry
	{
		uint32_t IntIpAddr;
		uint32_t IntPort;
		uint32_t IntProt;
		uint32_t ExtIpAddr;
		uint32_t ExtPort;
	} APE_ENTRY;


typedef struct apeList
	{
	uint32_t count;         /* Filled by the user */
	uint32_t fill_count;     /* Filled by upnpd*/
	APE_ENTRY entries[0];  
	} APE_LIST;


/* forward declarations */

#include "../../mgmt/upnpMgmt.c"

/*******************************************************************************
*
* main - main for upnp configuration tool
*
* NOMANUAL
* 
* RETURNS: N/A
*/
int main 
    (
    int argc,        /* number of arguments */
    char ** argv   /* argument vectors */
    )
    {
    char *                      pProgName;
    int                         offsetToHash=0;
    sqlite3 *                   pDb = NULL;
    SQLITE_BUSY_HANDLER_ARG     bhArg = {"", 100, 100};
    int status;

    /* search for the command corresponding to this program name */
    pProgName = argv[0];
    while (*pProgName)
        {
        if (*pProgName == '/')
            offsetToHash = (pProgName - argv[0]) + 1;
        pProgName++;
        }
    pProgName = argv[0] + offsetToHash;

    /* for ADP_DEBUG macros */
    umiInit(); 

    if (strcmp (pProgName , "upnpConfig") == 0)
        {
        if(argc >= 5)
            return upnpMgmtDBUpdateHandler (argv[1], 
                                             atoi(argv[2]), 
                                             argv[3], atoi(argv[4]),
                                             &argv[5],argc-5);
        ADP_PRINTF ("Usage:%s <DBFile> <opType> <tblName> <rowId>\n", 
                    argv[0]);
        return (ERROR);

        }
    /* open database connection */
    if (sqlite3_open (argv[1], &pDb) != SQLITE_OK)
        {
        ADP_PRINTF ("Could not open database: %s\n", sqlite3_errmsg(pDb));
        return ERROR;
        }
    strcpy(bhArg.progName, pProgName);
    sqlite3_busy_handler (pDb, sqliteBusyHandler, &bhArg);

    if (strcmp (pProgName , "upnpShowConfig") == 0)
        {
        if (argc != 2)
            {
            ADP_PRINTF ("Usage:%s <DBFile>\n", pProgName);
            if (pDb)
            sqlite3_close (pDb);
            return (ERROR);
            }
        status = upnpPortMap (pDb);
        }
    else
        {
        ADP_PRINTF("Illegal invocation of upnpConfig (%s)", pProgName);
        return (ERROR);
        }
    if (pDb)
        sqlite3_close (pDb);
    return status;
    }





