/* ipt_TPROXYRST.c -  */

/* Copyright (c) 2010, TeamF1, Inc. */

/*
modification history
--------------------
01a,21jan10,kri  written
*/

#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/checksum.h>

#include <linux/netfilter/x_tables.h>
#include <linux/netfilter_ipv4/ipt_TPROXYRST.h>

MODULE_LICENSE("TEAMF1");
MODULE_AUTHOR("Krishna B <krishnab@teamf1.com>");
MODULE_DESCRIPTION("ip[4]tables TPROXYRST module unsets TPROXY");
MODULE_ALIAS("ipt_TPROXYRST");

/* Functionality of TPROXYRST target */
static unsigned int
ipt_tproxyrst_target (struct sk_buff **pskb,
	  const struct net_device *in,
	  const struct net_device *out,
	  unsigned int hooknum,
	  const struct xt_target *target,
	  const void *targinfo)
{
    struct sk_buff *skb;
	const struct ipt_tproxyrst_target_info *markinfo = 
        (struct ipt_tproxyrst_target_info *)targinfo;
    skb = *pskb;

    /* unsetting of mark value bits*/ 
	skb->mark = skb->mark  & ~ (markinfo->mark & markinfo->mask);

    /* Removing tproxy ownership socket reference set by tproxy target
     * No need to put NULL check on skb->sk skb_orphan will take care */
    skb_orphan(skb);

	return XT_CONTINUE;
}


/* Check routine for TPROXYRST target data*/
static int
ipt_tproxyrst_checkentry(const char *tablename,
	      const void *entry,
	      const struct xt_target *target,
	      void *targinfo,
	      unsigned int hook_mask)
{
	struct ipt_tproxyrst_target_info *markinfo = 
        (struct ipt_tproxyrst_target_info *)targinfo;

	if ((markinfo->mark > MAX_32BIT_VALUE) || 
                (markinfo->mask > MAX_32BIT_VALUE)) 
    {
		printk(KERN_WARNING "MARK: Only supports 32bit wide mark/mask\n");
		return false;
	}

	return true;
}


/* netfilter TPROXYRST target structure */
static struct xt_target ipt_tproxyrst_tg = {
	
		.name		= "TPROXYRST",
		.family		= AF_INET,
		.checkentry	= ipt_tproxyrst_checkentry,
		.target		= ipt_tproxyrst_target,
		.targetsize	= XT_ALIGN(sizeof(struct ipt_tproxyrst_target_info)),
		.table		= "tproxy",
		.me		= THIS_MODULE,
};

static int __init ipt_tproxyrst_init(void)
{
    /* Register TPROXYRST target with netfilter */
	return xt_register_target(&ipt_tproxyrst_tg);
}

static void __exit ipt_tproxyrst_fini(void)
{
    /* Unregister TPROXYRST target with netfilter */
	xt_unregister_target(&ipt_tproxyrst_tg);
}

module_init(ipt_tproxyrst_init);
module_exit(ipt_tproxyrst_fini);
