/* wanAdvanced.js - API library for WAN Advanced setting configuration */

/* Copyright 2010 TeamF1, Inc. */

/*
modification history 
------------------------
01a, 22Aug11, Smita P, Modified functions to accomodate Failure Detection Method 
and Upload/Download Settings sections
*/

function wanAdvValidate ()
    {
    var txtFieldIdArr = new Array ();    
    txtFieldIdArr[0] = "txtMTUSizeDefault, Invalid MTU size: Please enter a valid MTU size";
    txtFieldIdArr[1] = "txtRouterMACAddr, Invalid MAC address: Please enter a value in the form XX:XX:XX:XX:XX:XX";
    txtFieldIdArr[2] = "txtRetryTime, Invalid Retry Interval: Please enter a valid Retry Interval";
	txtFieldIdArr[3] = "txtRetryAttempts, Invalid Failover tries: Please enter a valid Failover tries";
	txtFieldIdArr[4] = "txtUpCustom, Invalid WAN Connection Speed Upload: Please enter a valid Upload Speed";
	txtFieldIdArr[5] = "txtDownCustom, Invalid WAN Connection Speed Download: Please enter a valid Download Speed";

    if (txtFieldArrayCheck (txtFieldIdArr) == false)
         return false;

	if (isProblemCharArrayCheck (txtFieldIdArr) == false)
         return false;
 	if (mtuSizeValid() == false)
    	return false;
       var macObj = document.getElementById('txtRouterMACAddr');
	if (macObj && !macObj.disabled) 
    	{
    		if (macAddrValidate (macObj.value, true, 'Invalid MAC address:', '') == false)
        	{
        	macObj.focus ();
        	return false;
        	}	
        } 
           
	if (ipv4AddrOctetsValidate ('tblWi1DnsIp', 'IPD', false, true, 'Invalid DNS server IP address.', 'for octet ', true) == false){
        return false;}
        
    if (ipv4AddrOctetsValidate ('tblWi1PingIp', 'IPD', false, true, 'Invalid IP address.', 'for octet ', true) == false)
        return false;
	
    var retryObj = document.getElementById('txtRetryTime');
    if (retryObj && !retryObj.disabled) 
    	{
    	if (numericValueRangeCheck (retryObj, 1, 'Invalid Retry Interval.', 30, 999, true, 'Invalid Retry Interval.', 'Seconds') == false)
        	return false;
        }

    retryAttemptsObj = document.getElementById('txtRetryAttempts');
    if (retryAttemptsObj || !retryAttemptsObj.disabled)
    	{
	    if (numericValueRangeCheck (retryAttemptsObj, 1, 'Invalid Failover tries', 2, 999, true, 'Invalid Failover tries', '') == false)
	    	return false;
	    }

    return true;
    }
    
function mtuSizeValid()
	{
	var loginReqd = document.getElementById("hdLoginReqd").value;
	var maxMtu = 1500;
	if(loginReqd == "1")
		maxMtu = 1492;
	var mtuSizeObj = document.getElementById('txtMTUSizeDefault');
    if (mtuSizeObj && !mtuSizeObj.disabled)
    	{
    	if (numericValueRangeCheck (mtuSizeObj, 1, 'Invalid MTU Size:', 68, maxMtu, true, 'Invalid MTU Size:', '') == false) 
        	return false;
        }    return true;
	}
	
function wanAdvancedInit ()
	{
	MTUSizeCheck ();
	routerMACAddrCheck ();
	wanFailureDetMethod ();
	enableCustomSpeed ('selUpload', 'txtUpCustom');
	enableCustomSpeed ('selDownload', 'txtDownCustom');
	checkFailOver ();
//	setConnectionSpeed ();
	}

function MTUSizeCheck ()
	{
	var selValue = radioCheckedValueGet ("Wan.DefMtu");
	if (!selValue) return;
	switch (parseInt(selValue, 10))
		{
		case 0:	/* Custom */
			fieldStateChangeWr ('','','txtMTUSizeDefault','');
			break;
		case 1:	/* Default */
			fieldStateChangeWr ('txtMTUSizeDefault','','','');
			break;
		}
	}
function routerMACAddrCheck ()
	{
	var selValue = radioCheckedValueGet ("Wan.UseMac");
	if (!selValue) return;
	switch (parseInt(selValue, 10))
		{
		case 0:	/* Use Default Address */
			fieldStateChangeWr ('txtRouterMACAddr','','','');
			break;
		case 1:	/* Use this computer's MAC */
			fieldStateChangeWr ('txtRouterMACAddr','','','');
			break;
		case 2:	/* Use this MAC Address */
			fieldStateChangeWr ('','','txtRouterMACAddr','');
			break;
		}
	}
	
function wanFailureDetMethod ()
	{
	var method = comboSelectedValueGet  ('selDetectType');
	if (!method ) return;
	switch (parseInt(method , 10))
		{
		case 1:		/* USE WAN DNS */
			fieldStateChangeWr ('','tblWi1DnsIp tblWi1PingIp','txtRetryTime txtRetryAttempts','');
			break;
		case 2:		/* Custom DNS */
			fieldStateChangeWr ('','tblWi1PingIp','txtRetryTime txtRetryAttempts','tblWi1DnsIp');
			break;
		case 3:		/* Ping */
			fieldStateChangeWr ('','tblWi1DnsIp','txtRetryTime txtRetryAttempts','tblWi1PingIp');
			break;
		case 0:		/* None */
		default:
			fieldStateChangeWr ('txtRetryTime txtRetryAttempts','tblWi1DnsIp tblWi1PingIp','','');
			break;
	
		}  
	}

function enableCustomSpeed (selSpeed, txtCustom)
	{
	var selVal = comboSelectedValueGet (selSpeed);
    if (selVal == 0)
    	fieldStateChangeWr ('','',txtCustom,'');
    else	
    	fieldStateChangeWr (txtCustom,'','','');
	}
   
function setConnectionSpeed ()
	{
	var selValue = comboSelectedValueGet ('selConnType');
	var upSpeedObj = document.getElementById('selUpload');
	var downSpeedObj = document.getElementById('selDownload');
	if (!selValue || !upSpeedObj || !downSpeedObj) return;
    switch (selValue)
    	{
        case 'DSL':
        case 'CABLE':
        	upSpeedObj.selectedIndex = 7;
        	downSpeedObj.selectedIndex = 7;
        	break;
        case 'ADSL':
        	upSpeedObj.selectedIndex = 4;
        	downSpeedObj.selectedIndex = 7;
        	break;
        case 'T1':
        	upSpeedObj.selectedIndex = 8;
        	downSpeedObj.selectedIndex = 8;
        	break;
        case 'T3':
        	upSpeedObj.selectedIndex = 10;
        	downSpeedObj.selectedIndex = 10;
        	break;
        case 'Other':
        default:
        	upSpeedObj.selectedIndex = 1;
        	downSpeedObj.selectedIndex = 1;
    	}
    fieldStateChangeWr ('txtUpCustom txtDownCustom','','','');
	}
	
function checkFailOver ()
	{
	var autoRollOverObj = document.getElementById ('hdAutoRollOver');
	if (!autoRollOverObj) return;
	if (autoRollOverObj.value != "1") return;
	var primWanObj = document.getElementById ('hdPrimWanId');
	var wanIdObj = document.getElementById ('hdWanId');
	if (!primWanObj || !wanIdObj) return;
	if (primWanObj.value == wanIdObj.value)
		{
		var selObj = document.getElementById ('selDetectType');
		deleteOptions (selObj)
		selObj.options [0] = new Option("WAN DNS", "0", false, false);
		selObj.options [1] = new Option("Custom DNS", "1", false, false);
		selObj.options [2] = new Option("Ping", "2", false, false);
		optionValueSelect ('selDetectType', 'hdFailDetect');
		wanFailureDetMethod ();
		}
	}

